@@ -1,22 +1,25 @@
 use strict;
 use warnings;
-package JSON::Schema::Modern::Document::OpenAPI; # git description: 35b0dbd
+package JSON::Schema::Modern::Document::OpenAPI; # git description: v0.001-15-g67f9fe8
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
-# ABSTRACT: One JSON Schema document
+# ABSTRACT: One OpenAPI v3.1 document
 # KEYWORDS: JSON Schema data validation request response OpenAPI
 
-our $VERSION = '0.001';
+our $VERSION = '0.002';
 
-use 5.016;
+use 5.020;  # for fc, unicode_strings features
+use Moo;
+use strictures 2;
+use experimental qw(signatures postderef);
 use if "$]" >= 5.022, experimental => 're_strict';
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
-use JSON::Schema::Modern::Utilities 0.524 qw(assert_keyword_exists assert_keyword_type E);
+use JSON::Schema::Modern::Utilities 0.525 qw(assert_keyword_exists assert_keyword_type E get_type canonical_uri);
 use Safe::Isa;
 use File::ShareDir 'dist_dir';
 use Path::Tiny;
-use Moo;
+use List::Util 'any';
 use Types::Standard 'InstanceOf';
 use namespace::clean;
 
@@ -48,11 +51,10 @@ has json_schema_dialect => (
   coerce => sub { $_[0]->$_isa('Mojo::URL') ? $_[0] : Mojo::URL->new($_[0]) },
 );
 
-sub traverse {
-  my ($self, $evaluator) = @_;
+sub traverse ($self, $evaluator) {
+  $self->_add_vocab_and_default_schemas;
 
   my $schema = $self->schema;
-
   my $state = {
     initial_schema_uri => $self->canonical_uri,
     traversed_schema_path => '',
@@ -66,69 +68,81 @@ sub traverse {
     vocabularies => [],
   };
 
-  # /openapi
+  # /openapi: https://spec.openapis.org/oas/v3.1.0#openapi-object
   return $state if not assert_keyword_exists({ %$state, keyword => 'openapi' }, $schema)
     or not assert_keyword_type({ %$state, keyword => 'openapi' }, $schema, 'string');
 
-  my $valid = 1;
-  $valid = E({ %$state, keyword => 'openapi' }, 'unrecognized openapi version %s', $schema->{openapi})
-    if $schema->{openapi} !~ /^3\.1\.[0-9]+(-.+)?$/;
-
-  # /info -> /info/title, /info/version
-  {
-    my $state = { %$state, schema_path => $state->{schema_path}.'/info', keyword => 'info' };
-
-    return $state if not assert_keyword_exists($state, $schema)
-      or not assert_keyword_type($state, $schema, 'object')
-      or not grep
-        assert_keyword_exists($state, $schema)
-          && assert_keyword_type({ %$state, keyword => $_, schema_path => '/info' }, $schema->{info}, 'string'),
-        qw(title version);
-  }
-
-  # /jsonSchemaDialect
-
-  return $state if exists $schema->{jsonSchemaDialect}
-    and not assert_keyword_type({ %$state, keyword => 'jsonSchemaDialect' }, $schema, 'string');
-
-  my $json_schema_dialect = $self->json_schema_dialect // $schema->{jsonSchemaDialect};
-
-  if (not $json_schema_dialect) {
-    # "If [jsonSchemaDialect] is not set, then the OAS dialect schema id MUST be used for these Schema Objects."
-    $self->_add_vocab_and_default_schemas;
-    $json_schema_dialect = DEFAULT_DIALECT;
-  }
-
-  $self->_set_json_schema_dialect($json_schema_dialect);
-
-  # traverse an empty schema with this metaschema uri to confirm it is valid
-  my $check_metaschema_state = $evaluator->traverse({}, {
-    metaschema_uri => $json_schema_dialect,
-    initial_schema_uri => $self->canonical_uri->clone->fragment('/jsonSchemaDialect'),
-  });
-
-  @{$state}{qw(spec_version vocabularies)} = @{$check_metaschema_state}{qw(spec_version vocabularies)};
-
-  if (@{$check_metaschema_state->{errors}}) {
-    push @{$state->{errors}}, @{$check_metaschema_state->{errors}};
+  if ($schema->{openapi} !~ /^3\.1\.[0-9]+(-.+)?$/) {
+    ()= E({ %$state, keyword => 'openapi' }, 'unrecognized openapi version %s', $schema->{openapi});
     return $state;
   }
 
+
+  # /jsonSchemaDialect: https://spec.openapis.org/oas/v3.1.0#specifying-schema-dialects
+  {
+    return $state if exists $schema->{jsonSchemaDialect}
+      and not assert_keyword_type({ %$state, keyword => 'jsonSchemaDialect' }, $schema, 'string');
+
+    my $json_schema_dialect = $self->json_schema_dialect // $schema->{jsonSchemaDialect};
+
+    # "If [jsonSchemaDialect] is not set, then the OAS dialect schema id MUST be used for these Schema Objects."
+    $json_schema_dialect //= DEFAULT_DIALECT;
+
+    # traverse an empty schema with this metaschema uri to confirm it is valid
+    my $check_metaschema_state = $evaluator->traverse({}, {
+      metaschema_uri => $json_schema_dialect,
+      initial_schema_uri => $self->canonical_uri->clone->fragment('/jsonSchemaDialect'),
+    });
+
+    # we cannot continue if the metaschema is invalid
+    if ($check_metaschema_state->{errors}->@*) {
+      push $state->{errors}->@*, $check_metaschema_state->{errors}->@*;
+      return $state;
+    }
+
+    $state->@{qw(spec_version vocabularies)} = $check_metaschema_state->@{qw(spec_version vocabularies)};
+    $self->_set_json_schema_dialect($json_schema_dialect);
+  }
+
+  # evaluate the document against its metaschema to find any errors, and to identify all schema
+  # resources within to add to the global resource index.
+  my @json_schema_paths;
+  my $result = $self->evaluator->evaluate(
+    $self->schema,
+    $self->metaschema_uri,
+    {
+      callbacks => {
+        '$dynamicRef' => sub ($schema, $state) {
+          push @json_schema_paths, $state->{data_path} if $schema->{'$dynamicRef'} eq '#meta';
+        },
+      },
+    },
+  );
+
+  if (not $result) {
+    push $state->{errors}->@*, $result->errors;
+    return $state;
+  }
+
+  my @real_json_schema_paths;
+  foreach my $path (sort @json_schema_paths) {
+    # disregard paths that are not the root of each embedded subschema.
+    next if any { $_->subsumes($path) } @real_json_schema_paths;
+
+    push @real_json_schema_paths, path($path);
+    $self->_traverse_schema($self->get($path), { %$state, schema_path => $path });
+  }
+
   return $state;
 }
 
-before validate =>sub {
-  my $self = shift;
-  $self->_add_vocab_and_default_schemas;
-};
-
-sub _add_vocab_and_default_schemas {
-  my $self = shift;
+######## NO PUBLIC INTERFACES FOLLOW THIS POINT ########
 
+sub _add_vocab_and_default_schemas ($self) {
   my $js = $self->evaluator;
   $js->add_vocabulary('JSON::Schema::Modern::Vocabulary::OpenAPI');
 
-  foreach my $filename (keys %{(DEFAULT_SCHEMAS)}) {
+  foreach my $filename (keys DEFAULT_SCHEMAS->%*) {
     $js->add_schema(
       DEFAULT_SCHEMAS->{$filename},
       $js->_json_decoder->decode(path(dist_dir('JSON-Schema-Modern-Document-OpenAPI'), $filename)->slurp_raw),
@@ -137,6 +151,24 @@ sub _add_vocab_and_default_schemas {
   }
 }
 
+# https://spec.openapis.org/oas/v3.1.0#schema-object
+sub _traverse_schema ($self, $schema, $state) {
+  my $schema_type = get_type($schema);
+  return 1 if $schema_type eq 'boolean' or $schema_type eq 'object' and not keys %$schema;
+
+  my $subschema_state = $self->evaluator->traverse($schema, {
+    %$state,  # so we don't have to ennumerate everything that may be in config_override
+    initial_schema_uri => canonical_uri($state),
+    traversed_schema_path => $state->{traversed_schema_path}.$state->{schema_path},
+    metaschema_uri => $self->json_schema_dialect,
+  });
+
+  push $state->{errors}->@*, $subschema_state->{errors}->@*;
+  return if @{$subschema_state->{errors}};
+
+  push $state->{identifiers}->@*, $subschema_state->{identifiers}->@*;
+}
+
 1;
 
 __END__
@@ -147,11 +179,11 @@ __END__
 
 =head1 NAME
 
-JSON::Schema::Modern::Document::OpenAPI - One JSON Schema document
+JSON::Schema::Modern::Document::OpenAPI - One OpenAPI v3.1 document
 
 =head1 VERSION
 
-version 0.001
+version 0.002
 
 =head1 SYNOPSIS
 
@@ -215,8 +247,7 @@ longer be assumed.
 
 =head1 SUPPORT
 
-Bugs may be submitted through L<the RT bug tracker|https://rt.cpan.org/Public/Dist/Display.html?Name=JSON-Schema-Modern-Document-OpenAPI>
-(or L<bug-JSON-Schema-Modern-Document-OpenAPI@rt.cpan.org|mailto:bug-JSON-Schema-Modern-Document-OpenAPI@rt.cpan.org>).
+Bugs may be submitted through L<https://github.com/karenetheridge/JSON-Schema-Modern-Document-OpenAPI/issues>.
 
 I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.libera.chat>.
 
