@@ -4,16 +4,17 @@ package JSON::Schema::Modern::Vocabulary::OpenAPI;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema OpenAPI vocabulary
 
-our $VERSION = '0.001';
+our $VERSION = '0.002';
 
-use 5.016;
+use 5.020;  # for fc, unicode_strings features
+use Moo;
+use strictures 2;
+use experimental qw(signatures postderef);
 use if "$]" >= 5.022, experimental => 're_strict';
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
-use strictures 2;
 use JSON::Schema::Modern::Utilities 0.524 qw(assert_keyword_type annotate_self E is_type is_uri_reference jsonp);
-use Moo;
 use namespace::clean;
 
 with 'JSON::Schema::Modern::Vocabulary';
@@ -23,15 +24,10 @@ sub vocabulary {
 }
 
 sub keywords {
-  my ($self, $spec_version) = @_;
-  return (
-    qw(discriminator example externalDocs xml),
-  );
+  qw(discriminator example externalDocs xml);
 }
 
-sub _traverse_keyword_discriminator {
-  my ($self, $schema, $state) = @_;
-
+sub _traverse_keyword_discriminator ($self, $schema, $state) {
   return if not assert_keyword_type($state, $schema, 'object');
 
   # "the discriminator field MUST be a required field"
@@ -45,7 +41,7 @@ sub _traverse_keyword_discriminator {
     return if not assert_keyword_type({ %$state, _schema_path_suffix => 'mapping' }, $schema, 'object');
     return E({ %$state, _schema_path_suffix => 'mapping' }, 'discriminator mapping is not an object ')
       if not is_type('object', $schema->{discriminator}{mapping});
-    foreach my $mapping_key (sort keys %{$schema->{discriminator}{mapping}}) {
+    foreach my $mapping_key (sort keys $schema->{discriminator}{mapping}->%*) {
       my $uri = $schema->{discriminator}{mapping}{$mapping_key};
       $valid = E({ %$state, _schema_path_suffix => [ 'mapping', $mapping_key ] }, 'discriminator mapping value for "%s" is not a string', $mapping_key), next if not is_type('string', $uri);
       $valid = E({ %$state, _schema_path_suffix => [ 'mapping', $mapping_key ] }, 'discriminator mapping value for "%s" is not a uri-reference', $mapping_key), next if not is_uri_reference($uri);
@@ -58,9 +54,7 @@ sub _traverse_keyword_discriminator {
   return 1;
 }
 
-sub _eval_keyword_discriminator {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_discriminator ($self, $data, $schema, $state) {
   # Note: the spec is unclear of the expected behaviour when the data instance is not an object
   return 1 if not is_type('object', $data);
 
@@ -96,8 +90,7 @@ sub _eval_keyword_discriminator {
 
 sub _traverse_keyword_example { 1 }
 
-sub _eval_keyword_example {
-  my ($self, $data, $schema, $state) = @_;
+sub _eval_keyword_example ($self, $data, $schema, $state) {
   annotate_self($state, $schema);
 }
 
@@ -125,7 +118,7 @@ JSON::Schema::Modern::Vocabulary::OpenAPI - Implementation of the JSON Schema Op
 
 =head1 VERSION
 
-version 0.001
+version 0.002
 
 =head1 DESCRIPTION
 
@@ -142,8 +135,7 @@ L<https://spec.openapis.org/oas/3.1/dialect/base>.
 
 =head1 SUPPORT
 
-Bugs may be submitted through L<the RT bug tracker|https://rt.cpan.org/Public/Dist/Display.html?Name=JSON-Schema-Modern-Document-OpenAPI>
-(or L<bug-JSON-Schema-Modern-Document-OpenAPI@rt.cpan.org|mailto:bug-JSON-Schema-Modern-Document-OpenAPI@rt.cpan.org>).
+Bugs may be submitted through L<https://github.com/karenetheridge/JSON-Schema-Modern-Document-OpenAPI/issues>.
 
 I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.libera.chat>.
 
